@@ -1,6 +1,6 @@
 /* shmbutil.h -- utility functions for multibyte characters. */
 
-/* Copyright (C) 2002-2004 Free Software Foundation, Inc.
+/* Copyright (C) 2002-2019 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -29,14 +29,16 @@
 #if defined (HANDLE_MULTIBYTE)
 #include "shmbchar.h"
 
-extern size_t xmbsrtowcs __P((wchar_t *, const char **, size_t, mbstate_t *));
-extern size_t xdupmbstowcs __P((wchar_t **, char ***, const char *));
+extern size_t xwcsrtombs PARAMS((char *, const wchar_t **, size_t, mbstate_t *));
+extern size_t xmbsrtowcs PARAMS((wchar_t *, const char **, size_t, mbstate_t *));
+extern size_t xdupmbstowcs PARAMS((wchar_t **, char ***, const char *));
 
-extern size_t mbstrlen __P((const char *));
+extern size_t mbstrlen PARAMS((const char *));
 
-extern char *xstrchr __P((const char *, int));
+extern char *xstrchr PARAMS((const char *, int));
 
 extern int locale_mb_cur_max;	/* XXX */
+extern int locale_utf8locale;	/* XXX */
 
 #ifndef MB_INVALIDCH
 #define MB_INVALIDCH(x)		((x) == (size_t)-1 || (x) == (size_t)-2)
@@ -49,6 +51,10 @@ extern int locale_mb_cur_max;	/* XXX */
 #define MBLEN(s, n)	((MB_CUR_MAX > 1) ? mblen ((s), (n)) : 1)
 #define MBRLEN(s, n, p)	((MB_CUR_MAX > 1) ? mbrlen ((s), (n), (p)) : 1)
 
+#define UTF8_SINGLEBYTE(c)	(((c) & 0x80) == 0)
+#define UTF8_MBFIRSTCHAR(c)	(((c) & 0xc0) == 0xc0)
+#define UTF8_MBCHAR(c)		(((c) & 0xc0) == 0x80)
+
 #else /* !HANDLE_MULTIBYTE */
 
 #undef MB_LEN_MAX
@@ -74,6 +80,9 @@ extern int locale_mb_cur_max;	/* XXX */
 #  define wchar_t	int
 #endif
 
+#define UTF8_SINGLEBYTE(c)	(1)
+#define UTF8_MBFIRSTCHAR(c)	(0)
+
 #endif /* !HANDLE_MULTIBYTE */
 
 /* Declare and initialize a multibyte state.  Call must be terminated
@@ -109,6 +118,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    _f = is_basic ((_str)[_i]); \
 	    if (_f) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && (((_str)[_i] & 0x80) == 0)) \
+	      mblength = (_str)[_i] != 0; \
 	    else \
 	      { \
 	        state_bak = state; \
@@ -149,6 +160,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    _f = is_basic (*(_str)); \
 	    if (_f) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && ((*(_str) & 0x80) == 0)) \
+	      mblength = *(_str) != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -267,6 +280,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    _k = is_basic (*(_src)); \
 	    if (_k) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && ((*(_src) & 0x80) == 0)) \
+	      mblength = *(_src) != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -303,9 +318,11 @@ extern int locale_mb_cur_max;	/* XXX */
 	    size_t mblength; \
 	    int _k; \
 \
-	    _k = is_basic (*((_src) + (_si))); \
+	    _k = is_basic ((_src)[(_si)]); \
 	    if (_k) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && ((_src)[(_si)] & 0x80) == 0) \
+	      mblength = (_src)[(_si)] != 0; \
 	    else \
 	      {\
 		state_bak = state; \
@@ -346,9 +363,11 @@ extern int locale_mb_cur_max;	/* XXX */
 	    size_t mblength; \
 	    int _i; \
 \
-	    _i = is_basic (*((_src) + (_si))); \
+	    _i = is_basic ((_src)[(_si)]); \
 	    if (_i) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && ((_src)[(_si)] & 0x80) == 0) \
+	      mblength = (_src)[(_si)] != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -396,6 +415,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    _i = is_basic (*((_src) + (_si))); \
 	    if (_i) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && (((_src)[_si] & 0x80) == 0)) \
+              mblength = (_src)[_si] != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -440,6 +461,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    i = is_basic (*((_src) + (_si))); \
 	    if (i) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && (((_src)[_si] & 0x80) == 0)) \
+	      mblength = (_src)[_si] != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -478,6 +501,8 @@ extern int locale_mb_cur_max;	/* XXX */
 	    i = is_basic (*((_src) + (_si))); \
 	    if (i) \
 	      mblength = 1; \
+	    else if (locale_utf8locale && (((_src)[_si] & 0x80) == 0)) \
+	      mblength = (_src)[_si] != 0; \
 	    else \
 	      { \
 		state_bak = state; \
@@ -508,7 +533,9 @@ extern int locale_mb_cur_max;	/* XXX */
 	    i = is_basic (*((_src) + (_si))); \
 	    if (i) \
 	      mblength = 1; \
-	    else \
+	    else if (locale_utf8locale && (((_src)[_si] & 0x80) == 0)) \
+              mblength = (_src)[_si] != 0; \
+            else \
 	      { \
 		state_bak = state; \
 		mblength = mbrlen ((_src) + (_si), (_srcsize) - (_si), &state); \
