@@ -1,6 +1,6 @@
 /* nls.c -- skeletal internationalization code. */
 
-/* Copyright (C) 1996-2009 Free Software Foundation, Inc.
+/* Copyright (C) 1996-2022 Free Software Foundation, Inc.
 
    This file is part of the GNU Readline Library (Readline), a library
    for reading lines of text with interactive input and history editing.      
@@ -53,8 +53,12 @@
 #include "readline.h"
 #include "rlshell.h"
 #include "rlprivate.h"
+#include "xmalloc.h"
 
-static int utf8locale PARAMS((char *));
+static int utf8locale (char *);
+
+#define RL_DEFAULT_LOCALE "C"
+static char *_rl_current_locale = 0;
 
 #if !defined (HAVE_SETLOCALE)    
 /* A list of legal values for the LANG or LC_CTYPE environment variables.
@@ -74,19 +78,19 @@ static char *legal_lang_values[] =
  "iso88599",
  "iso885910",
  "koi8r",
+ "utf8",
   0
 };
 
-static char *normalize_codeset PARAMS((char *));
+static char *normalize_codeset (char *);
 #endif /* !HAVE_SETLOCALE */
 
-static char *find_codeset PARAMS((char *, size_t *));
+static char *find_codeset (char *, size_t *);
 
-static char *_rl_get_locale_var PARAMS((const char *));
+static char *_rl_get_locale_var (const char *);
 
 static char *
-_rl_get_locale_var (v)
-     const char *v;
+_rl_get_locale_var (const char *v)
 {
   char *lspec;
 
@@ -100,8 +104,7 @@ _rl_get_locale_var (v)
 }
 
 static int
-utf8locale (lspec)
-     char *lspec;
+utf8locale (char *lspec)
 {
   char *cp;
   size_t len;
@@ -118,16 +121,12 @@ utf8locale (lspec)
 #endif
 }
 
-/* Check for LC_ALL, LC_CTYPE, and LANG and use the first with a value
-   to decide the defaults for 8-bit character input and output.  Returns
-   1 if we set eight-bit mode. */
-int
-_rl_init_eightbit ()
+/* Query the right environment variables and call setlocale() to initialize
+   the C library locale settings. */
+char *
+_rl_init_locale (void)
 {
-/* If we have setlocale(3), just check the current LC_CTYPE category
-   value, and go into eight-bit mode if it's not C or POSIX. */
-#if defined (HAVE_SETLOCALE)
-  char *lspec, *t;
+  char *ret, *lspec;
 
   /* Set the LC_CTYPE locale category from environment variables. */
   lspec = _rl_get_locale_var ("LC_CTYPE");
@@ -136,35 +135,65 @@ _rl_init_eightbit ()
      that doesn't return anything, we set lspec to the empty string to
      force the subsequent call to setlocale() to define the `native'
      environment. */
+#if defined (HAVE_SETLOCALE)
   if (lspec == 0 || *lspec == 0)
     lspec = setlocale (LC_CTYPE, (char *)NULL);
   if (lspec == 0)
     lspec = "";
-  t = setlocale (LC_CTYPE, lspec);
+  ret = setlocale (LC_CTYPE, lspec);	/* ok, since it does not change locale */
+  if (ret == 0 || *ret == 0)
+    ret = setlocale (LC_CTYPE, (char *)NULL);
+  if (ret == 0 || *ret == 0)
+    ret = RL_DEFAULT_LOCALE;
+#else
+  ret = (lspec == 0 || *lspec == 0) ? RL_DEFAULT_LOCALE : lspec;
+#endif
 
-  if (t && *t)
-    _rl_utf8locale = utf8locale (t);
+  _rl_utf8locale = (ret && *ret) ? utf8locale (ret) : 0;
 
-  if (t && *t && (t[0] != 'C' || t[1]) && (STREQ (t, "POSIX") == 0))
+  _rl_current_locale = savestring (ret);
+  return ret;
+}
+
+/* If we have setlocale(3), just check the current LC_CTYPE category
+   value (passed as LOCALESTR), and go into eight-bit mode if it's not "C"
+   or "POSIX". If FORCE is non-zero, we reset the locale variables to values
+   appropriate for the C locale if the locale is "C" or "POSIX". FORCE is 0
+   when this is called from _rl_init_eightbit, since we're modifying the
+   default initial values and don't need to change anything else. If we
+   don't have setlocale(3), we check the codeset portion of LOCALESTR against
+   a set of known values and go into eight-bit mode if it matches one of those.
+   Returns 1 if we set eight-bit (multibyte) mode. */
+static int
+_rl_set_localevars (char *localestr, int force)
+{
+#if defined (HAVE_SETLOCALE)
+  if (localestr && *localestr && (localestr[0] != 'C' || localestr[1]) && (STREQ (localestr, "POSIX") == 0))
     {
       _rl_meta_flag = 1;
       _rl_convert_meta_chars_to_ascii = 0;
       _rl_output_meta_chars = 1;
       return (1);
     }
+  else if (force)
+    {
+      /* Default "C" locale settings. */
+      _rl_meta_flag = 0;
+      _rl_convert_meta_chars_to_ascii = 1;
+      _rl_output_meta_chars = 0;
+      return (0);
+    }
   else
     return (0);
 
 #else /* !HAVE_SETLOCALE */
-  char *lspec, *t;
+  char *t;
   int i;
 
   /* We don't have setlocale.  Finesse it.  Check the environment for the
      appropriate variables and set eight-bit mode if they have the right
      values. */
-  lspec = _rl_get_locale_var ("LC_CTYPE");
-
-  if (lspec == 0 || (t = normalize_codeset (lspec)) == 0)
+  if (localestr == 0 || (t = normalize_codeset (localestr)) == 0)
     return (0);
   for (i = 0; t && legal_lang_values[i]; i++)
     if (STREQ (t, legal_lang_values[i]))
@@ -174,16 +203,40 @@ _rl_init_eightbit ()
 	_rl_output_meta_chars = 1;
 	break;
       }
+
+  if (force && legal_lang_values[i] == 0)	/* didn't find it */
+    {
+      /* Default "C" locale settings. */
+      _rl_meta_flag = 0;
+      _rl_convert_meta_chars_to_ascii = 1;
+      _rl_output_meta_chars = 0;
+    }
+
+  _rl_utf8locale = *t ? STREQ (t, "utf8") : 0;
+
   xfree (t);
   return (legal_lang_values[i] ? 1 : 0);
-
 #endif /* !HAVE_SETLOCALE */
 }
 
+/* Check for LC_ALL, LC_CTYPE, and LANG and use the first with a value
+   to decide the defaults for 8-bit character input and output.  Returns
+   1 if we set eight-bit mode. */
+int
+_rl_init_eightbit (void)
+{
+  char *t, *ol;
+
+  ol = _rl_current_locale;
+  t = _rl_init_locale ();	/* resets _rl_current_locale, returns static pointer */
+  xfree (ol);
+
+  return (_rl_set_localevars (t, 0));
+}
+
 #if !defined (HAVE_SETLOCALE)
 static char *
-normalize_codeset (codeset)
-     char *codeset;
+normalize_codeset (char *codeset)
 {
   size_t namelen, i;
   int len, all_digits;
@@ -230,9 +283,7 @@ normalize_codeset (codeset)
 
 /* Isolate codeset portion of locale specification. */
 static char *
-find_codeset (name, lenp)
-     char *name;
-     size_t *lenp;
+find_codeset (char *name, size_t *lenp)
 {
   char *cp, *language, *result;
 
@@ -279,3 +330,19 @@ find_codeset (name, lenp)
 
   return result;
 }
+
+void
+_rl_reset_locale (void)
+{
+  char *ol, *nl;
+
+  /* This should not be NULL; _rl_init_eightbit sets it on the first call to
+     readline() or rl_initialize(). */
+  ol = _rl_current_locale;
+  nl = _rl_init_locale ();		/* resets _rl_current_locale */
+
+  if ((ol == 0 && nl) || (ol && nl && (STREQ (ol, nl) == 0)))
+    (void)_rl_set_localevars (nl, 1);
+
+  xfree (ol);
+}
